# 1 "test_rpe_var.pf"
# 1 "<built-in>"
# 1 "<command-line>"
# 1 "/usr/include/stdc-predef.h" 1 3 4
# 1 "<command-line>" 2
# 1 "test_rpe_var.pf"
! Copyright 2015-2016 Andrew Dawson, Peter Dueben
!
! Licensed under the Apache License, Version 2.0 (the "License");
! you may not use this file except in compliance with the License.
! You may obtain a copy of the License at
!
! http:
!
! Unless required by applicable law or agreed to in writing, software
! distributed under the License is distributed on an "AS IS" BASIS,
! WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
! See the License for the specific language governing permissions and
! limitations under the License.

MODULE test_rpe_var
! Tests for the `rpe_var` type.
!
    USE pfunit_mod
    USE suite_common, ONLY : utest64_t, utest64, utest32, utest32_t
    USE rp_emulator
    IMPLICIT NONE

# 1 "../include/emutest_type.pf" 1
!@TESTCASE
TYPE, EXTENDS(TestCase) :: EmulatorTest
    LOGICAL :: rpe_active
    INTEGER :: rpe_default_sbits
    LOGICAL :: rpe_ieee_half
CONTAINS
    procedure :: setUp
    procedure :: tearDown
END TYPE EmulatorTest
# 24 "test_rpe_var.pf" 2

CONTAINS

# 1 "../include/emutest_proc.pf" 1
SUBROUTINE setUp (this)
    CLASS(EmulatorTest), INTENT(INOUT) :: this
    this%rpe_active = RPE_ACTIVE
    this%rpe_default_sbits = RPE_DEFAULT_SBITS
    this%rpe_ieee_half = RPE_IEEE_HALF
END SUBROUTINE setUp

SUBROUTINE tearDown (this)
    CLASS(EmulatorTest), INTENT(INOUT) :: this
    RPE_ACTIVE = this%rpe_active
    RPE_DEFAULT_SBITS = this%rpe_default_sbits
    RPE_IEEE_HALF = this%rpe_ieee_half
END SUBROUTINE tearDown
# 28 "test_rpe_var.pf" 2

    !@TEST
    SUBROUTINE test_rpe_var_assign_reduce_default (context)
    ! Verify that the precision of a value assigned to an `rpe_var`
    ! instance is reduced to the default number of bits by the
    ! assignment operator when no precision level has been set.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced

        reduced = utest64
#line 69 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, utest64_t(52), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 69) )
  if (anyExceptions()) return
#line 70 "test_rpe_var.pfp"

    END SUBROUTINE test_rpe_var_assign_reduce_default

    !@TEST
    SUBROUTINE test_rpe_var_assign_reduce_specified (context)
    ! Verify that the precision of a value assigned to an `rpe_var`
    ! instance is reduced to the instance's specified number of bits by
    ! the assignment operator.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced
        INTEGER :: nbits

        DO nbits = 1, 52
            reduced%sbits = nbits
            reduced = utest64
#line 86 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, utest64_t(nbits), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 86) )
  if (anyExceptions()) return
#line 87 "test_rpe_var.pfp"
        END DO

    END SUBROUTINE test_rpe_var_assign_reduce_specified

    !@TEST
    SUBROUTINE test_rpe_literal_real_64bit_explicit (context)
    ! Test that one can construct am rpe_var instance from a real number
    ! with kind RPE_REAL_KIND.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced
        INTEGER :: nbits

        DO nbits = 1, 52
            reduced = rpe_literal(utest64, nbits)
#line 102 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, utest64_t(nbits), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 102) )
  if (anyExceptions()) return
#line 103 "test_rpe_var.pfp"
        END DO

    END SUBROUTINE test_rpe_literal_real_64bit_explicit

    !@TEST
    SUBROUTINE test_rpe_literal_real_64bit_defaults (context)
    ! Test that one can construct am rpe_var instance from a real number
    ! with kind RPE_REAL_KIND.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced
        INTEGER :: nbits

        DO nbits = 1, 52
            RPE_DEFAULT_SBITS = nbits
            reduced = rpe_literal(utest64)
#line 119 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, utest64_t(nbits), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 119) )
  if (anyExceptions()) return
#line 120 "test_rpe_var.pfp"
        END DO

    END SUBROUTINE test_rpe_literal_real_64bit_defaults

    !@TEST
    SUBROUTINE test_rpe_literal_real_32bit_explicit (context)
    ! Test that one can construct am rpe_var instance from a real number
    ! with kind RPE_REAL_KIND.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced
        INTEGER :: nbits

        DO nbits = 1, 23
            reduced = rpe_literal(utest32, nbits)
#line 135 "test_rpe_var.pfp"
  call assertEQUAL(utest32_t(nbits), reduced%val, &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 135) )
  if (anyExceptions()) return
#line 136 "test_rpe_var.pfp"
        END DO

    END SUBROUTINE test_rpe_literal_real_32bit_explicit

    !@TEST
    SUBROUTINE test_rpe_literal_real_32bit_defaults (context)
    ! Test that one can construct am rpe_var instance from a real number
    ! with kind RPE_REAL_KIND.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced
        INTEGER :: nbits

        DO nbits = 1, 23
            RPE_DEFAULT_SBITS = nbits
            reduced = rpe_literal(utest32)
#line 152 "test_rpe_var.pfp"
  call assertEQUAL(utest32_t(nbits), reduced%val, &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 152) )
  if (anyExceptions()) return
#line 153 "test_rpe_var.pfp"
        END DO

    END SUBROUTINE test_rpe_literal_real_32bit_defaults

    !@TEST
    SUBROUTINE test_rpe_literal_integer_explicit (context)
    ! Test that one can construct am rpe_var instance from a real number
    ! with kind RPE_REAL_KIND.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced
        INTEGER :: exact_value, inexact_value

        exact_value = 1024 ! exact at 23-bits
        reduced = rpe_literal(exact_value, 23)
#line 168 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, REAL(exact_value, RPE_REAL_KIND), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 168) )
  if (anyExceptions()) return
#line 169 "test_rpe_var.pfp"
        inexact_value = 10241024 ! = 10256384 at 8-bits
        reduced = rpe_literal(inexact_value, 8)
#line 171 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, REAL(10256384, RPE_REAL_KIND), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 171) )
  if (anyExceptions()) return
#line 172 "test_rpe_var.pfp"

    END SUBROUTINE test_rpe_literal_integer_explicit

    !@TEST
    SUBROUTINE test_rpe_literal_integer_defaults (context)
    ! Test that one can construct am rpe_var instance from a real number
    ! with kind RPE_REAL_KIND.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced
        INTEGER :: exact_value, inexact_value

        exact_value = 1024 ! exact at 23-bits
        RPE_DEFAULT_SBITS = 23
        reduced = rpe_literal(exact_value)
#line 187 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, REAL(exact_value, RPE_REAL_KIND), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 187) )
  if (anyExceptions()) return
#line 188 "test_rpe_var.pfp"
        inexact_value = 10241024 ! = 10256384 at 8-bits
        RPE_DEFAULT_SBITS = 8
        reduced = rpe_literal(inexact_value)
#line 191 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, REAL(10256384, RPE_REAL_KIND), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 191) )
  if (anyExceptions()) return
#line 192 "test_rpe_var.pfp"

    END SUBROUTINE test_rpe_literal_integer_defaults

    !@TEST
    SUBROUTINE test_rpe_literal_long_explicit (context)
    ! Test that one can construct am rpe_var instance from a real number
    ! with kind RPE_REAL_KIND.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced
        INTEGER(KIND=8) :: exact_value, inexact_value

        exact_value = 1024 ! exact at 23-bits
        reduced = rpe_literal(exact_value, 23)
#line 206 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, REAL(exact_value, RPE_REAL_KIND), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 206) )
  if (anyExceptions()) return
#line 207 "test_rpe_var.pfp"
        inexact_value = 10241024 ! = 10256384 at 8-bits
        reduced = rpe_literal(inexact_value, 8)
#line 209 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, REAL(10256384, RPE_REAL_KIND), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 209) )
  if (anyExceptions()) return
#line 210 "test_rpe_var.pfp"

    END SUBROUTINE test_rpe_literal_long_explicit

    !@TEST
    SUBROUTINE test_rpe_literal_long_defaults (context)
    ! Test that one can construct am rpe_var instance from a real number
    ! with kind RPE_REAL_KIND.
    !
        CLASS(EmulatorTest), INTENT(INOUT) :: context
        TYPE(rpe_var) :: reduced
        INTEGER(KIND=8) :: exact_value, inexact_value

        exact_value = 1024 ! exact at 23-bits
        RPE_DEFAULT_SBITS = 23
        reduced = rpe_literal(exact_value)
#line 225 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, REAL(exact_value, RPE_REAL_KIND), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 225) )
  if (anyExceptions()) return
#line 226 "test_rpe_var.pfp"
        inexact_value = 10241024 ! = 10256384 at 8-bits
        RPE_DEFAULT_SBITS = 8
        reduced = rpe_literal(inexact_value)
#line 229 "test_rpe_var.pfp"
  call assertEQUAL(reduced%val, REAL(10256384, RPE_REAL_KIND), &
 & location=SourceLocation( &
 & 'test_rpe_var.pfp', &
 & 229) )
  if (anyExceptions()) return
#line 230 "test_rpe_var.pfp"

    END SUBROUTINE test_rpe_literal_long_defaults

END MODULE test_rpe_var

module Wraptest_rpe_var
   use pFUnit_mod
   use test_rpe_var
   implicit none
   private

   public :: WrapUserTestCase
   public :: makeCustomTest
   type, extends(EmulatorTest) :: WrapUserTestCase
      procedure(userTestMethod), nopass, pointer :: testMethodPtr
   contains
      procedure :: runMethod
   end type WrapUserTestCase

   abstract interface
     subroutine userTestMethod(this)
        use test_rpe_var
        class (EmulatorTest), intent(inout) :: this
     end subroutine userTestMethod
   end interface

contains

   subroutine runMethod(this)
      class (WrapUserTestCase), intent(inout) :: this

      call this%testMethodPtr(this)
   end subroutine runMethod

   function makeCustomTest(methodName, testMethod) result(aTest)
#ifdef INTEL_13
      use pfunit_mod, only: testCase
#endif
      type (WrapUserTestCase) :: aTest
#ifdef INTEL_13
      target :: aTest
      class (WrapUserTestCase), pointer :: p
#endif
      character(len=*), intent(in) :: methodName
      procedure(userTestMethod) :: testMethod
      aTest%testMethodPtr => testMethod
#ifdef INTEL_13
      p => aTest
      call p%setName(methodName)
#else
      call aTest%setName(methodName)
#endif
   end function makeCustomTest

end module Wraptest_rpe_var

function test_rpe_var_suite() result(suite)
   use pFUnit_mod
   use test_rpe_var
   use Wraptest_rpe_var
   type (TestSuite) :: suite

   integer, allocatable :: npes(:)

   suite = newTestSuite('test_rpe_var_suite')

   call suite%addTest(makeCustomTest('test_rpe_var_assign_reduce_default', test_rpe_var_assign_reduce_default))

   call suite%addTest(makeCustomTest('test_rpe_var_assign_reduce_specified', test_rpe_var_assign_reduce_specified))

   call suite%addTest(makeCustomTest('test_rpe_literal_real_64bit_explicit', test_rpe_literal_real_64bit_explicit))

   call suite%addTest(makeCustomTest('test_rpe_literal_real_64bit_defaults', test_rpe_literal_real_64bit_defaults))

   call suite%addTest(makeCustomTest('test_rpe_literal_real_32bit_explicit', test_rpe_literal_real_32bit_explicit))

   call suite%addTest(makeCustomTest('test_rpe_literal_real_32bit_defaults', test_rpe_literal_real_32bit_defaults))

   call suite%addTest(makeCustomTest('test_rpe_literal_integer_explicit', test_rpe_literal_integer_explicit))

   call suite%addTest(makeCustomTest('test_rpe_literal_integer_defaults', test_rpe_literal_integer_defaults))

   call suite%addTest(makeCustomTest('test_rpe_literal_long_explicit', test_rpe_literal_long_explicit))

   call suite%addTest(makeCustomTest('test_rpe_literal_long_defaults', test_rpe_literal_long_defaults))


end function test_rpe_var_suite

